from pybrary.contract import Contract, contract

from pybrary.func import fqn


def expect(obj, name):
    assert fqn(obj)==name, f'\n ! expected {name}\n !      got {fqn(obj)}'


def test_n_fqn():
    expect(contract, 'pybrary.contract.contract')
    expect(Contract, 'pybrary.contract.Contract')
    expect(Contract.pre, 'pybrary.contract.pre')
    expect(Contract(expect), 'pybrary.contract.Contract')
    expect(Contract(expect).pre, 'pybrary.contract.pre')


def test_c_fqn():
    from os import listdir
    from sys import path
    expect(42, 'int')
    expect({}, 'dict')
    expect(listdir, 'listdir')
    expect(path, 'list')


if __name__=='__main__': test_c_fqn()



from pathlib import Path
from os import makedirs

from pybrary.files import (
    files,
    find,
    grep,
    FileStat,
)


dest = '/tmp/test'

def init():
    dest_a = f'{dest}/aaa'
    dest_b = f'{dest}/bbb'
    for dst in (dest_a, dest_b):
        makedirs(dst, exist_ok=True)
        for t in 'abc':
            with open(f'{dst}/test_{t}', 'w') as out:
                out.write('abc')
        for tt in 'toto titi tata'.split():
            with open(f'{dst}/{tt}', 'w') as out:
                out.write(tt)


def test_n_files():
    init()

    expected = [
        f'{dest}/aaa/tata',
        f'{dest}/aaa/test_a',
        f'{dest}/aaa/test_b',
        f'{dest}/aaa/test_c',
        f'{dest}/aaa/titi',
        f'{dest}/aaa/toto',
        f'{dest}/bbb/tata',
        f'{dest}/bbb/test_a',
        f'{dest}/bbb/test_b',
        f'{dest}/bbb/test_c',
        f'{dest}/bbb/titi',
        f'{dest}/bbb/toto',
    ]

    found = []
    for f in files(dest):
        assert isinstance(f, Path), f'{type(f)} : {f}'
        assert f.is_file()
        found.append(str(f))

    assert found == expected


def test_n_find():
    init()

    expected = [
        f'{dest}/aaa/tata',
        f'{dest}/aaa/titi',
        f'{dest}/aaa/toto',
        f'{dest}/bbb/tata',
        f'{dest}/bbb/titi',
        f'{dest}/bbb/toto',
    ]

    found = []
    for f in find(dest, r't.t.'):
        found.append(str(f))

    assert found == expected


def test_n_grep():
    init()

    for f in files(dest):
        for g in grep(f, '(a.c)'):
            assert g[0] == 'abc'

    for f in files(dest):
        for g in grep(f, 't(.)t(.)'):
            assert g[0] == g[1]
            assert f't{g[0]}t{g[1]}' in str(f)


def test_n_age():
    path = '/tmp/tmp'
    with open(path, 'w'): pass
    age = FileStat(path).age
    assert age==0


if __name__=='__main__': test_n_age()



from functools import reduce
from operator import __add__

from pytest import raises

from pybrary.contract import (
    Contract,
    contract,
    PreError,
    ResultError,
    PostError,
)


class Adder(Contract):
    def pre(self, *a, **k):
        assert not k, "Adder doesn't support keywords arguments"
        assert len(a)>1, "Adder needs at least 2 arguments"
        for n in a:
            assert isinstance(n, (int, float)), f'invalid arg {type(n).__name__}({n}) for {self.name}, must be a number'

    def alt(self, *a, **k):
        r = 0
        for n in a: r+=n
        return r

    def post(self, r, *a, **k):
        if all(isinstance(n, int) for n in a):
            assert isinstance(r, int), f'wrong {self.name} result type : {type(r).__name__}, expected int'
        else:
            assert isinstance(r, float), f'wrong {self.name} result type : {type(r).__name__}, expected float'


def test_n_adder():
    @contract(Adder)
    def add1(*a):
        return sum(a)

    @contract(Adder)
    def add2(*a):
        return reduce(__add__, a)

    for add in (add1, add2):
        add(1, 2, 3)


def test_f_adder_arg():
    @contract(Adder)
    def add(*a):
        return sum(a)

    with raises(PreError): add(a=1)
    with raises(PreError): add(1)
    with raises(PreError): add(1, '2')


def test_f_adder_result():
    @contract(Adder)
    def add(*a):
        return sum(a)+1

    with raises(ResultError): add(2, 2)


def test_f_adder_post():
    @contract(Adder)
    def add(*a):
        return int(sum(a))

    with raises(PostError): add(2, 2.0)


if __name__=='__main__': test_f_adder_post()



from pytest import raises

from pybrary.func import (
    caller,
    todo,
    memo,
)


def callee():
    return caller()

def test_n_caller():
    assert callee() == 'test_func.test_n_caller'


def test_n_todo():
    class AbsCls:
        def abs_meth(self): todo(self)

    a = AbsCls()
    with raises(NotImplementedError) as x:
        a.abs_meth()
    assert str(x.value).strip() == '! abs_meth missing in AbsCls !'

def test_n_memo():
    class MemoTest:
        @memo
        def prop(self):
            return 'value'

    c = MemoTest()
    assert not hasattr(c, '_memo_prop')
    assert c.prop == c._memo_prop


if __name__=='__main__': test_n_memo()



from sys import argv

import pybrary


def parse_argv():
    cmd, fct, *opt = argv
    args, kw = [], {}
    for o in opt:
        if '=' in p:
            k, v = o.split('=')
            kw[k] = v
        else:
            args.append(o)
    return cmd, fct, args, kw


def get_ip_adr():
    try:
        ok, adr = pybrary.get_ip_adr()
        if ok:
            print(adr)
        else:
            print('ERROR : %s' % adr)
    except Exception as x:
        print('Exception ! %s' % x)



from pybrary.ascii import rm_ansi_codes, oct_mod


def test_s_oct_mod():
    assert oct_mod('r--r--r--') == '444'


def test_s_rm_ansi():
    esc, color, txt = chr(27), 33, 'OK'
    colored = f'{esc}[1;{color}m{txt}{esc}[0m'

    assert rm_ansi_codes(colored) == 'OK'


if __name__=='__main__': test_s_rm_ansi()



from urllib.request import urlopen


def get_ip_adr():
    url = 'http://api.ipify.org'
    with urlopen(url, timeout=1) as resp:
        if resp.status==200:
            adr = resp.read().decode().strip()
            return True, adr
        else:
            return False, resp.reason



from pybrary.html import Parser


def test_n_parser():
    url = "https://pypi.org/search?q=pybrary"
    root = Parser().get(url)
    name = version = None
    for pkg in (n for n in root if 'package-snippet' in n.classes):
        for node in (n for n in pkg if 'package-snippet__name' in n.classes):
            if node.text=='pybrary':
                name = node.text
                # print(f'{name = }')
                for ver in (n for n in pkg if 'package-snippet__version' in n.classes):
                    version = ver.text
                    # print(f'{version = }')

    assert name=='pypbrary'
    assert version


if __name__=='__main__': test_n_parser()
